Package: mwcsr
Title: Solvers for Maximum Weight Connected Subgraph Problem and Its
        Variants
Version: 0.1.11
Authors@R: c(person("Alexander", "Loboda", email = "aleks.loboda@gmail.com", role = c("aut", "cre")),
    person("Nikolay", "Poperechnyi", email = "n.poperechnyi@gmail.com", role = "aut"),
    person("Eduardo", "Alvarez-Miranda", email = "ealvarez@utalca.cl", role = "aut"), 
    person("Markus", "Sinnl", email = "markus.sinnl@univie.ac.at", role = "aut"),
    person("Alexey", "Sergushichev", email = "alsergbox@gmail.com", role = "aut"),
    person("Paul", "Hosler Jr.", role = "cph", comment = "Bundled JOpt Simple package"),
    person("www.hamcrest.org", role = "cph", comment = "Bundled hamcrest package"),
    person("Barak Naveh and Contributors", role = "cph", comment = "Bundled JGraphT package"),
    person("The Apache Software Foundation", role = "cph", comment = "Bundled Apache Commons Math package"))
Description: Algorithms for solving various Maximum
    Weight Connected Subgraph Problems, including variants with
    budget constraints, cardinality constraints, weighted edges and signals.
    The package represents an R interface to high-efficient solvers based on
    relax-and-cut approach (Álvarez-Miranda E., Sinnl M. (2017) <doi:10.1016/j.cor.2017.05.015>)
    mixed-integer programming (Loboda A., Artyomov M., and Sergushichev A. (2016) <doi:10.1007/978-3-319-43681-4_17>)
    and simulated annealing.
Depends: R (>= 3.5)
Imports: methods, igraph, Rcpp
Suggests: knitr, rmarkdown, mathjaxr, testthat, BioNet, roxygen2, DLBCL
SystemRequirements: Java (>=8)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr, rmarkdown
URL: https://github.com/ctlab/mwcsr
BugReports: https://github.com/ctlab/mwcsr/issues
LinkingTo: Rcpp
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-11 18:48:57 UTC; root
Author: Alexander Loboda [aut, cre],
  Nikolay Poperechnyi [aut],
  Eduardo Alvarez-Miranda [aut],
  Markus Sinnl [aut],
  Alexey Sergushichev [aut],
  Paul Hosler Jr. [cph] (Bundled JOpt Simple package),
  www.hamcrest.org [cph] (Bundled hamcrest package),
  Barak Naveh and Contributors [cph] (Bundled JGraphT package),
  The Apache Software Foundation [cph] (Bundled Apache Commons Math
    package)
Maintainer: Alexander Loboda <aleks.loboda@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 20:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-11 20:32:22 UTC; unix
Archs: mwcsr.so.dSYM
