# Generated by roxygen2: do not edit by hand

S3method("[",mwcs_solver)
S3method("[<-",mwcs_solver)
S3method(check_parameter,char_parameter)
S3method(check_parameter,default)
S3method(check_parameter,file_parameter)
S3method(check_parameter,float_parameter)
S3method(check_parameter,integer_parameter)
S3method(check_parameter,logical_parameter)
S3method(check_parameter,mc_parameter)
S3method(parameters,default)
S3method(parameters,rmwcs_solver)
S3method(parameters,rnc_solver)
S3method(parameters,scipjack_solver)
S3method(parameters,simulated_annealing_solver)
S3method(parameters,virgo_solver)
S3method(print,mwcs_solver)
S3method(print,mwcs_solver_params)
S3method(solve_mwcsp,rmwcs_solver)
S3method(solve_mwcsp,rnc_solver)
S3method(solve_mwcsp,scipjack_solver)
S3method(solve_mwcsp,simulated_annealing_solver)
S3method(solve_mwcsp,virgo_solver)
export("timelimit<-")
export(annealing_solver)
export(get_instance_type)
export(get_weight)
export(normalize_sgmwcs_instance)
export(parameters)
export(rmwcs_solver)
export(rnc_solver)
export(scipjack_solver)
export(set_parameters)
export(solve_mwcsp)
export(virgo_solver)
import(igraph)
importFrom(Rcpp,sourceCpp)
importFrom(methods,is)
importFrom(stats,setNames)
useDynLib(mwcsr)
