# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

sa_solve <- function(instance, solver) {
    .Call('_mwcsr_sa_solve', PACKAGE = 'mwcsr', instance, solver)
}

sgmwcs_solve <- function(instance, solver_params) {
    .Call('_mwcsr_sgmwcs_solve', PACKAGE = 'mwcsr', instance, solver_params)
}

rmwcs_solve <- function(network, params) {
    .Call('_mwcsr_rmwcs_solve', PACKAGE = 'mwcsr', network, params)
}

